@@ -1,7 +1,9 @@
 /*
- * Copyright (C) 2022 Guido Günther
+ * Copyright (C) 2022,2025 The Phosh Developers
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
+ *
+ * Author: Guido Günther <agx@sigxcpu.org>
  */
 
 #include <glib/gi18n.h>
@@ -9,10 +11,12 @@
 #include "mobile-settings-config.h"
 #include "mobile-settings-application.h"
 
+
 int
 main (int argc, char *argv[])
 {
   g_autoptr (MobileSettingsApplication) app = NULL;
+  g_autoptr (GError) err = NULL;
   int ret;
 
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
