@@ -6,7 +6,7 @@
  * Author: Guido Günther <agx@sigxcpu.org>
  */
 
-#define G_LOG_DOMAIN "ms-plugin-list_box"
+#define G_LOG_DOMAIN "ms-plugin-list-box"
 
 #include "mobile-settings-config.h"
 
@@ -52,7 +52,7 @@ G_DEFINE_TYPE (MsPluginListBox, ms_plugin_list_box, ADW_TYPE_BIN)
 
 
 
-static AdwPreferencesWindow *
+static AdwPreferencesDialog *
 load_prefs_window (MsPluginListBox *self, const char *name)
 {
   GIOExtensionPoint *ep;
@@ -63,6 +63,7 @@ load_prefs_window (MsPluginListBox *self, const char *name)
   g_return_val_if_fail (ep, NULL);
 
   ext = g_io_extension_point_get_extension_by_name (ep, name);
+  g_return_val_if_fail (ext, NULL);
 
   g_debug ("Loading plugin %s", name);
   type = g_io_extension_get_type (ext);
@@ -74,7 +75,7 @@ static void
 open_plugin_prefs_activated (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   MsPluginListBox *self = MS_PLUGIN_LIST_BOX (data);
-  AdwPreferencesWindow *prefs;
+  AdwPreferencesDialog *prefs;
   GtkWindow *parent;
   g_autoptr (GError) error = NULL;
   g_autoptr (GKeyFile) keyfile = g_key_file_new ();
@@ -96,10 +97,9 @@ open_plugin_prefs_activated (GSimpleAction *action, GVariant *parameter, gpointe
   g_assert (parent);
 
   prefs = load_prefs_window (self, name);
-  g_return_if_fail (prefs);
+  g_return_if_fail (ADW_IS_DIALOG (prefs));
 
-  gtk_window_set_transient_for (GTK_WINDOW (prefs), parent);
-  gtk_window_present (GTK_WINDOW (prefs));
+  adw_dialog_present (ADW_DIALOG (prefs), GTK_WIDGET (parent));
 }
 
 
@@ -239,9 +239,8 @@ ms_plugin_list_box_scan_phosh_plugins (MsPluginListBox *self)
 {
   g_autoptr (GError) err = NULL;
   g_autoptr (GDir) dir = g_dir_open (MOBILE_SETTINGS_PHOSH_PLUGINS_DIR, 0, &err);
-  const char *filename;
   g_auto (GStrv) enabled_plugins = NULL;
-
+  const char *filename;
 
   if (dir == NULL) {
     g_warning ("Failed to read phosh plugins from " MOBILE_SETTINGS_PHOSH_PLUGINS_DIR ": %s",
